@@ -3,7 +3,7 @@ import inspect
 from io import StringIO
 import sys
 from uuid import UUID
-from typing import TYPE_CHECKING, Any, Dict, List, cast
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, cast
 
 from pydantic import BaseModel
 
@@ -27,6 +27,7 @@ from vellum.workflows.expressions.equals import EqualsExpression
 from vellum.workflows.expressions.greater_than import GreaterThanExpression
 from vellum.workflows.expressions.greater_than_or_equal_to import GreaterThanOrEqualToExpression
 from vellum.workflows.expressions.in_ import InExpression
+from vellum.workflows.expressions.is_error import IsErrorExpression
 from vellum.workflows.expressions.is_nil import IsNilExpression
 from vellum.workflows.expressions.is_not_nil import IsNotNilExpression
 from vellum.workflows.expressions.is_not_null import IsNotNullExpression
@@ -50,13 +51,14 @@ from vellum.workflows.references.execution_count import ExecutionCountReference
 from vellum.workflows.references.lazy import LazyReference
 from vellum.workflows.references.output import OutputReference
 from vellum.workflows.references.state_value import StateValueReference
+from vellum.workflows.references.trigger import TriggerAttributeReference
 from vellum.workflows.references.vellum_secret import VellumSecretReference
 from vellum.workflows.references.workflow_input import WorkflowInputReference
 from vellum.workflows.types.core import JsonArray, JsonObject
 from vellum.workflows.types.generics import is_workflow_class
 from vellum.workflows.utils.functions import compile_function_definition
 from vellum.workflows.utils.uuids import uuid4_from_hash
-from vellum_ee.workflows.display.utils.exceptions import UnsupportedSerializationException
+from vellum_ee.workflows.display.utils.exceptions import InvalidInputReferenceError, UnsupportedSerializationException
 from vellum_ee.workflows.server.virtual_file_loader import VirtualFileLoader
 
 if TYPE_CHECKING:
@@ -104,6 +106,8 @@ def convert_descriptor_to_operator(descriptor: BaseDescriptor) -> LogicalOperato
         return "doesNotBeginWith"
     elif isinstance(descriptor, DoesNotEndWithExpression):
         return "doesNotEndWith"
+    elif isinstance(descriptor, IsErrorExpression):
+        return "isError"
     elif isinstance(descriptor, (IsNullExpression, IsNilExpression, IsUndefinedExpression)):
         return "null"
     elif isinstance(descriptor, (IsNotNullExpression, IsNotNilExpression, IsNotUndefinedExpression)):
@@ -165,6 +169,7 @@ def _serialize_condition(
     if isinstance(
         condition,
         (
+            IsErrorExpression,
             IsNullExpression,
             IsNotNullExpression,
             IsNilExpression,
@@ -248,13 +253,11 @@ def serialize_key(key: Any) -> str:
         return str(key)
 
 
-# Sentinel value to indicate a value should be omitted from serialization
-_UNDEFINED_SENTINEL: JsonObject = {"__undefined__": True}
-
-
-def serialize_value(executable_id: UUID, display_context: "WorkflowDisplayContext", value: Any) -> JsonObject:
+def serialize_value(executable_id: UUID, display_context: "WorkflowDisplayContext", value: Any) -> Optional[JsonObject]:
     """
-    Serialize a value to a JSON object.
+    Serialize a value to a JSON object. Returns `None` if the value resolves to `undefined`.
+    This is safe because all valid values are a JSON object, including the `None` constant:
+    > `{"type": "CONSTANT_VALUE", "value": {"type": "JSON", "value": None}}`
 
     Args:
         executable_id: node id or workflow id
@@ -265,7 +268,7 @@ def serialize_value(executable_id: UUID, display_context: "WorkflowDisplayContex
         serialized value
     """
     if value is undefined:
-        return _UNDEFINED_SENTINEL
+        return None
 
     if isinstance(value, ConstantValueReference):
         return serialize_value(executable_id, display_context, value._value)
@@ -275,16 +278,24 @@ def serialize_value(executable_id: UUID, display_context: "WorkflowDisplayContex
         return serialize_value(executable_id, display_context, child_descriptor)
 
     if isinstance(value, WorkflowInputReference):
-        try:
-            workflow_input_display = display_context.global_workflow_input_displays[value]
-        except KeyError:
-            inputs_class_name = value.inputs_class.__name__
-            workflow_class_name = display_context.workflow_display_class.infer_workflow_class().__name__
-            raise UnsupportedSerializationException(
-                f"Inputs class '{inputs_class_name}' referenced during serialization of '{workflow_class_name}' "
-                f"without parameterizing this Workflow with this Inputs definition. Update your Workflow "
-                f"definition to '{workflow_class_name}(BaseWorkflow[{inputs_class_name}, BaseState])'."
-            )
+        if value not in display_context.global_workflow_input_displays:
+            workflow_inputs_class = display_context.workflow_display_class.infer_workflow_class().get_inputs_class()
+            if value.inputs_class != workflow_inputs_class:
+                inputs_class_name = value.inputs_class.__name__
+                workflow_class_name = display_context.workflow_display_class.infer_workflow_class().__name__
+                raise UnsupportedSerializationException(
+                    f"Inputs class '{inputs_class_name}' referenced during serialization of "
+                    f"'{workflow_class_name}' without parameterizing this Workflow with this Inputs definition. "
+                    f"Update your Workflow definition to "
+                    f"'{workflow_class_name}(BaseWorkflow[{inputs_class_name}, BaseState])'."
+                )
+            else:
+                raise InvalidInputReferenceError(
+                    message=f"Inputs class '{value.inputs_class.__qualname__}' has no attribute '{value.name}'",
+                    inputs_class_name=value.inputs_class.__qualname__,
+                    attribute_name=value.name,
+                )
+        workflow_input_display = display_context.global_workflow_input_displays[value]
         return {
             "type": "WORKFLOW_INPUT",
             "input_variable_id": str(workflow_input_display.id),
@@ -341,11 +352,22 @@ def serialize_value(executable_id: UUID, display_context: "WorkflowDisplayContex
             "node_id": str(node_class_display.node_id),
         }
 
+    if isinstance(value, TriggerAttributeReference):
+        # Generate trigger ID using the same hash formula as in base_workflow_display.py
+        trigger_class = value.trigger_class
+        trigger_id = uuid4_from_hash(trigger_class.__qualname__)
+
+        return {
+            "type": "TRIGGER_ATTRIBUTE",
+            "trigger_id": str(trigger_id),
+            "attribute_id": str(value.id),
+        }
+
     if isinstance(value, list):
         serialized_items = []
         for item in value:
             serialized_item = serialize_value(executable_id, display_context, item)
-            if serialized_item != _UNDEFINED_SENTINEL:
+            if serialized_item is not None:
                 serialized_items.append(serialized_item)
 
         if all(isinstance(item, dict) and item["type"] == "CONSTANT_VALUE" for item in serialized_items):
@@ -381,7 +403,7 @@ def serialize_value(executable_id: UUID, display_context: "WorkflowDisplayContex
         serialized_entries: List[Dict[str, Any]] = []
         for key, val in value.items():
             serialized_val = serialize_value(executable_id, display_context, val)
-            if serialized_val != _UNDEFINED_SENTINEL:
+            if serialized_val is not None:
                 serialized_entries.append(
                     {
                         "id": str(uuid4_from_hash(f"{executable_id}|{key}")),
@@ -447,7 +469,9 @@ def serialize_value(executable_id: UUID, display_context: "WorkflowDisplayContex
         if inputs:
             serialized_inputs = {}
             for param_name, input_ref in inputs.items():
-                serialized_inputs[param_name] = serialize_value(executable_id, display_context, input_ref)
+                serialized_input = serialize_value(executable_id, display_context, input_ref)
+                if serialized_input is not None:
+                    serialized_inputs[param_name] = serialized_input
 
             model_data = function_definition.model_dump()
             model_data["inputs"] = serialized_inputs
