@@ -25,10 +25,13 @@ from vellum.workflows.nodes.utils import get_unadorned_node, get_unadorned_port,
 from vellum.workflows.ports import Port
 from vellum.workflows.references import OutputReference, WorkflowInputReference
 from vellum.workflows.state.encoder import DefaultStateEncoder
+from vellum.workflows.triggers.manual import ManualTrigger
+from vellum.workflows.triggers.vellum_integration import VellumIntegrationTrigger
 from vellum.workflows.types.core import Json, JsonArray, JsonObject
 from vellum.workflows.types.generics import WorkflowType
 from vellum.workflows.types.utils import get_original_base
-from vellum.workflows.utils.uuids import uuid4_from_hash
+from vellum.workflows.utils.uuids import get_trigger_id, uuid4_from_hash
+from vellum.workflows.utils.vellum_variables import primitive_type_to_vellum_variable_type
 from vellum.workflows.vellum_client import create_vellum_client
 from vellum_ee.workflows.display.base import (
     EdgeDisplay,
@@ -37,9 +40,10 @@ from vellum_ee.workflows.display.base import (
     WorkflowInputsDisplay,
     WorkflowMetaDisplay,
     WorkflowOutputDisplay,
+    WorkflowTriggerType,
+    get_trigger_type_mapping,
 )
 from vellum_ee.workflows.display.editor.types import NodeDisplayData, NodeDisplayPosition
-from vellum_ee.workflows.display.exceptions import NodeValidationError
 from vellum_ee.workflows.display.nodes.base_node_display import BaseNodeDisplay
 from vellum_ee.workflows.display.nodes.get_node_display_class import get_node_display_class
 from vellum_ee.workflows.display.nodes.types import NodeOutputDisplay, PortDisplay
@@ -57,6 +61,7 @@ from vellum_ee.workflows.display.types import (
     WorkflowOutputDisplays,
 )
 from vellum_ee.workflows.display.utils.auto_layout import auto_layout_nodes
+from vellum_ee.workflows.display.utils.exceptions import UserFacingException
 from vellum_ee.workflows.display.utils.expressions import serialize_value
 from vellum_ee.workflows.display.utils.registry import register_workflow_display_class
 from vellum_ee.workflows.display.utils.vellum import infer_vellum_variable_type
@@ -175,21 +180,62 @@ class BaseWorkflowDisplay(Generic[WorkflowType]):
         serialized_nodes: Dict[UUID, JsonObject] = {}
         edges: JsonArray = []
 
-        # Add a single synthetic node for the workflow entrypoint
-        entrypoint_node_id = self.display_context.workflow_display.entrypoint_node_id
-        entrypoint_node_source_handle_id = self.display_context.workflow_display.entrypoint_node_source_handle_id
-        serialized_nodes[entrypoint_node_id] = {
-            "id": str(entrypoint_node_id),
-            "type": "ENTRYPOINT",
-            "inputs": [],
-            "data": {
-                "label": "Entrypoint Node",
-                "source_handle_id": str(entrypoint_node_source_handle_id),
-            },
-            "display_data": self.display_context.workflow_display.entrypoint_node_display.dict(),
-            "base": None,
-            "definition": None,
-        }
+        # Get all trigger edges from the workflow's subgraphs to check if trigger exists
+        trigger_edges = []
+        for subgraph in self._workflow.get_subgraphs():
+            trigger_edges.extend(list(subgraph.trigger_edges))
+
+        # Determine if we need an ENTRYPOINT node and what ID to use
+        # For ManualTrigger: create ENTRYPOINT node for backward compatibility
+        # For IntegrationTrigger only: no ENTRYPOINT node (use trigger ID directly in edges)
+        manual_trigger_edges = [edge for edge in trigger_edges if issubclass(edge.trigger_class, ManualTrigger)]
+        integration_trigger_edges = [
+            edge for edge in trigger_edges if issubclass(edge.trigger_class, VellumIntegrationTrigger)
+        ]
+
+        has_manual_trigger = len(manual_trigger_edges) > 0
+        has_only_integration_trigger = len(integration_trigger_edges) > 0 and not has_manual_trigger
+
+        entrypoint_node_id: Optional[UUID] = None
+        entrypoint_node_source_handle_id: Optional[UUID] = None
+
+        if has_manual_trigger:
+            # ManualTrigger: use trigger ID for ENTRYPOINT node (backward compatibility)
+            trigger_class = manual_trigger_edges[0].trigger_class
+            entrypoint_node_id = get_trigger_id(trigger_class)
+            entrypoint_node_source_handle_id = self.display_context.workflow_display.entrypoint_node_source_handle_id
+
+            # Add ENTRYPOINT node for ManualTrigger workflows
+            serialized_nodes[entrypoint_node_id] = {
+                "id": str(entrypoint_node_id),
+                "type": "ENTRYPOINT",
+                "inputs": [],
+                "data": {
+                    "label": "Entrypoint Node",
+                    "source_handle_id": str(entrypoint_node_source_handle_id),
+                },
+                "display_data": self.display_context.workflow_display.entrypoint_node_display.dict(),
+                "base": None,
+                "definition": None,
+            }
+        elif not trigger_edges:
+            # No trigger: use default ENTRYPOINT node (backward compatibility)
+            entrypoint_node_id = self.display_context.workflow_display.entrypoint_node_id
+            entrypoint_node_source_handle_id = self.display_context.workflow_display.entrypoint_node_source_handle_id
+
+            serialized_nodes[entrypoint_node_id] = {
+                "id": str(entrypoint_node_id),
+                "type": "ENTRYPOINT",
+                "inputs": [],
+                "data": {
+                    "label": "Entrypoint Node",
+                    "source_handle_id": str(entrypoint_node_source_handle_id),
+                },
+                "display_data": self.display_context.workflow_display.entrypoint_node_display.dict(),
+                "base": None,
+                "definition": None,
+            }
+        # else: has_only_integration_trigger - no ENTRYPOINT node needed
 
         # Add all the nodes in the workflows
         for node in self._workflow.get_all_nodes():
@@ -203,12 +249,20 @@ class BaseWorkflowDisplay(Generic[WorkflowType]):
                     self.display_context.add_validation_error(validation_error)
 
                 serialized_node = node_display.serialize(self.display_context)
-            except (NotImplementedError, NodeValidationError) as e:
+            except (NotImplementedError, UserFacingException) as e:
                 self.display_context.add_error(e)
                 self.display_context.add_invalid_node(node)
                 continue
 
-            serialized_nodes[node_display.node_id] = serialized_node
+            # Use wrapped node's ID as dict key for adornment wrappers to prevent overwrites
+            wrapped_node = get_wrapped_node(node)
+            if wrapped_node:
+                wrapped_node_display = self.display_context.node_displays[wrapped_node]
+                dict_key = wrapped_node_display.node_id
+            else:
+                dict_key = node_display.node_id
+
+            serialized_nodes[dict_key] = serialized_node
 
         synthetic_output_edges: JsonArray = []
         output_variables: JsonArray = []
@@ -328,26 +382,53 @@ class BaseWorkflowDisplay(Generic[WorkflowType]):
                 ValueError("Unable to serialize terminal nodes that are not referenced by workflow outputs.")
             )
 
-        # Add an edge for each edge in the workflow
-        for target_node, entrypoint_display in self.display_context.entrypoint_displays.items():
-            unadorned_target_node = get_unadorned_node(target_node)
-            # Skip edges to invalid nodes
-            if self._is_node_invalid(unadorned_target_node):
-                continue
+        # Add edges from trigger/entrypoint to first nodes
+        if has_only_integration_trigger:
+            # Use trigger ID directly as sourceNodeId (no ENTRYPOINT node)
+            trigger_class = integration_trigger_edges[0].trigger_class
+            trigger_id = get_trigger_id(trigger_class)
+            trigger_source_handle_id = trigger_id  # Use trigger ID as handle ID
 
-            target_node_display = self.display_context.node_displays[unadorned_target_node]
-            entrypoint_edge_dict: Dict[str, Json] = {
-                "id": str(entrypoint_display.edge_display.id),
-                "source_node_id": str(entrypoint_node_id),
-                "source_handle_id": str(entrypoint_node_source_handle_id),
-                "target_node_id": str(target_node_display.node_id),
-                "target_handle_id": str(target_node_display.get_trigger_id()),
-                "type": "DEFAULT",
-            }
-            display_data = self._serialize_edge_display_data(entrypoint_display.edge_display)
-            if display_data is not None:
-                entrypoint_edge_dict["display_data"] = display_data
-            edges.append(entrypoint_edge_dict)
+            for target_node, entrypoint_display in self.display_context.entrypoint_displays.items():
+                unadorned_target_node = get_unadorned_node(target_node)
+                # Skip edges to invalid nodes
+                if self._is_node_invalid(unadorned_target_node):
+                    continue
+
+                target_node_display = self.display_context.node_displays[unadorned_target_node]
+                trigger_edge_dict: Dict[str, Json] = {
+                    "id": str(entrypoint_display.edge_display.id),
+                    "source_node_id": str(trigger_id),
+                    "source_handle_id": str(trigger_source_handle_id),
+                    "target_node_id": str(target_node_display.node_id),
+                    "target_handle_id": str(target_node_display.get_trigger_id()),
+                    "type": "DEFAULT",
+                }
+                display_data = self._serialize_edge_display_data(entrypoint_display.edge_display)
+                if display_data is not None:
+                    trigger_edge_dict["display_data"] = display_data
+                edges.append(trigger_edge_dict)
+        elif entrypoint_node_id is not None:
+            # Traditional approach: edges from ENTRYPOINT node (backward compatibility)
+            for target_node, entrypoint_display in self.display_context.entrypoint_displays.items():
+                unadorned_target_node = get_unadorned_node(target_node)
+                # Skip edges to invalid nodes
+                if self._is_node_invalid(unadorned_target_node):
+                    continue
+
+                target_node_display = self.display_context.node_displays[unadorned_target_node]
+                entrypoint_edge_dict: Dict[str, Json] = {
+                    "id": str(entrypoint_display.edge_display.id),
+                    "source_node_id": str(entrypoint_node_id),
+                    "source_handle_id": str(entrypoint_node_source_handle_id),
+                    "target_node_id": str(target_node_display.node_id),
+                    "target_handle_id": str(target_node_display.get_trigger_id()),
+                    "type": "DEFAULT",
+                }
+                display_data = self._serialize_edge_display_data(entrypoint_display.edge_display)
+                if display_data is not None:
+                    entrypoint_edge_dict["display_data"] = display_data
+                edges.append(entrypoint_edge_dict)
 
         for (source_node_port, target_node), edge_display in self.display_context.edge_displays.items():
             unadorned_source_node_port = get_unadorned_port(source_node_port)
@@ -400,22 +481,110 @@ class BaseWorkflowDisplay(Generic[WorkflowType]):
             except Exception as e:
                 self.display_context.add_error(e)
 
-        return {
-            "workflow_raw_data": {
-                "nodes": cast(JsonArray, nodes_dict_list),
-                "edges": edges,
-                "display_data": self.display_context.workflow_display.display_data.dict(),
-                "definition": {
-                    "name": self._workflow.__name__,
-                    "module": cast(JsonArray, self._workflow.__module__.split(".")),
-                },
-                "output_values": output_values,
+        # Serialize workflow-level trigger if present
+        triggers: Optional[JsonArray] = self._serialize_workflow_trigger()
+
+        workflow_raw_data: JsonObject = {
+            "nodes": cast(JsonArray, nodes_dict_list),
+            "edges": edges,
+            "display_data": self.display_context.workflow_display.display_data.dict(),
+            "definition": {
+                "name": self._workflow.__name__,
+                "module": cast(JsonArray, self._workflow.__module__.split(".")),
             },
+            "output_values": output_values,
+        }
+
+        result: JsonObject = {
+            "workflow_raw_data": workflow_raw_data,
             "input_variables": input_variables,
             "state_variables": state_variables,
             "output_variables": output_variables,
         }
 
+        if triggers is not None:
+            result["triggers"] = triggers
+
+        return result
+
+    def _serialize_workflow_trigger(self) -> Optional[JsonArray]:
+        """
+        Serialize workflow-level trigger information.
+
+        Returns:
+            JsonArray with trigger data if a trigger is present, None otherwise.
+            Each trigger in the array has: id (UUID), type (str), attributes (list)
+        """
+        # Get all trigger edges from the workflow's subgraphs
+        trigger_edges = []
+        for subgraph in self._workflow.get_subgraphs():
+            trigger_edges.extend(list(subgraph.trigger_edges))
+
+        if not trigger_edges:
+            # No workflow-level trigger defined
+            return None
+
+        # Get the trigger class from the first edge
+        trigger_class = trigger_edges[0].trigger_class
+
+        # Validate that all trigger edges use the same trigger type
+        trigger_type_mapping = get_trigger_type_mapping()
+        for edge in trigger_edges:
+            if edge.trigger_class != trigger_class:
+                raise ValueError(
+                    f"Mixed trigger types not supported. Found {trigger_class.__name__} and "
+                    f"{edge.trigger_class.__name__} in the same workflow."
+                )
+
+        # Get the trigger type from the mapping, or check if it's a VellumIntegrationTrigger subclass
+        trigger_type = trigger_type_mapping.get(trigger_class)
+        if trigger_type is None:
+            # Check if it's a VellumIntegrationTrigger subclass
+
+            if issubclass(trigger_class, VellumIntegrationTrigger):
+                trigger_type = WorkflowTriggerType.INTEGRATION
+            else:
+                raise ValueError(
+                    f"Unknown trigger type: {trigger_class.__name__}. "
+                    f"Please add it to the trigger type mapping in get_trigger_type_mapping()."
+                )
+
+        # Return as a list with a single trigger object matching Django schema
+        trigger_id = get_trigger_id(trigger_class)
+
+        # Serialize trigger attributes like node outputs
+        attribute_references = trigger_class.attribute_references().values()
+        trigger_attributes: JsonArray = cast(
+            JsonArray,
+            [
+                cast(
+                    JsonObject,
+                    {
+                        "id": str(reference.id),
+                        "name": reference.name,
+                        "type": primitive_type_to_vellum_variable_type(reference),
+                        "value": None,
+                    },
+                )
+                for reference in sorted(attribute_references, key=lambda ref: ref.name)
+            ],
+        )
+
+        trigger_data: JsonObject = {
+            "id": str(trigger_id),
+            "type": trigger_type.value,
+            "attributes": trigger_attributes,
+        }
+
+        # For INTEGRATION triggers, include class name, module path, and source_handle_id
+        # ManualTrigger doesn't need source_handle_id because edges come from ENTRYPOINT node
+        if trigger_type == WorkflowTriggerType.INTEGRATION:
+            trigger_data["class_name"] = trigger_class.__name__
+            trigger_data["module_path"] = cast(Json, trigger_class.__module__.split("."))
+            trigger_data["source_handle_id"] = str(trigger_id)  # IntegrationTrigger acts as edge source
+
+        return cast(JsonArray, [trigger_data])
+
     def _serialize_edge_display_data(self, edge_display: EdgeDisplay) -> Optional[JsonObject]:
         """Serialize edge display data, returning None if no display data is present."""
         if edge_display.z_index is not None:
