@@ -1,6 +1,6 @@
 from typing import Any, ClassVar, Dict, Generic, Iterator, List, Optional, Set, Union
 
-from vellum import ChatMessage, PromptBlock
+from vellum import ChatMessage, PromptBlock, PromptOutput
 from vellum.client.types.prompt_parameters import PromptParameters
 from vellum.client.types.prompt_settings import PromptSettings
 from vellum.prompts.constants import DEFAULT_PROMPT_PARAMETERS
@@ -22,8 +22,8 @@ from vellum.workflows.nodes.displayable.tool_calling_node.utils import (
 )
 from vellum.workflows.outputs.base import BaseOutput, BaseOutputs
 from vellum.workflows.state.context import WorkflowContext
-from vellum.workflows.types.core import EntityInputsInterface, Tool
-from vellum.workflows.types.definition import MCPServer
+from vellum.workflows.types.core import EntityInputsInterface
+from vellum.workflows.types.definition import MCPServer, Tool
 from vellum.workflows.types.generics import StateType
 from vellum.workflows.utils.functions import compile_mcp_tool_definition, get_mcp_tool_name
 from vellum.workflows.workflows.event_filters import all_workflow_event_filter
@@ -47,7 +47,7 @@ class ToolCallingNode(BaseNode[StateType], Generic[StateType]):
     functions: ClassVar[List[Tool]] = []
     prompt_inputs: ClassVar[Optional[EntityInputsInterface]] = None
     parameters: PromptParameters = DEFAULT_PROMPT_PARAMETERS
-    max_prompt_iterations: ClassVar[Optional[int]] = 5
+    max_prompt_iterations: ClassVar[Optional[int]] = 25
     settings: ClassVar[Optional[Union[PromptSettings, Dict[str, Any]]]] = None
 
     class Outputs(BaseOutputs):
@@ -82,6 +82,7 @@ class ToolCallingNode(BaseNode[StateType], Generic[StateType]):
                 class Outputs(BaseWorkflow.Outputs):
                     text: str = self.tool_prompt_node.Outputs.text
                     chat_history: List[ChatMessage] = ToolCallingState.chat_history
+                    results: List[PromptOutput] = self.tool_prompt_node.Outputs.results
 
             subworkflow = ToolCallingWorkflow(
                 parent_state=self.state,
@@ -97,9 +98,6 @@ class ToolCallingNode(BaseNode[StateType], Generic[StateType]):
         exception: Optional[NodeException] = None
         fulfilled_output_names: Set[str] = set()
 
-        # Yield initiated event for chat_history
-        yield BaseOutput(name="chat_history")
-
         for event in subworkflow_stream:
             self._context._emit_subworkflow_event(event)
 
@@ -123,6 +121,17 @@ class ToolCallingNode(BaseNode[StateType], Generic[StateType]):
                 exception = NodeException.of(event.error)
 
         if exception:
+            # In the case of the Prompt Node receiving invalid inputs or schemas, that is a bug with the
+            # internals of the Agent Node. In the future, we should distinguish further by also checking
+            # that this exception came from the Tool Calling Node's prompt node instead of anything
+            # internal to a user defined tool.
+            if exception.code == WorkflowErrorCode.INVALID_INPUTS:
+                raise NodeException(
+                    message="Internal server error",
+                    code=WorkflowErrorCode.INTERNAL_ERROR,
+                    raw_data=exception.raw_data,
+                    stacktrace=exception.stacktrace,
+                ) from exception
             raise exception
 
         if outputs is None:
