@@ -31,12 +31,14 @@ from vellum.workflows.outputs.base import BaseOutput
 from vellum.workflows.ports.port import Port
 from vellum.workflows.state import BaseState
 from vellum.workflows.state.encoder import DefaultStateEncoder
-from vellum.workflows.types.core import EntityInputsInterface, MergeBehavior, Tool, ToolBase
+from vellum.workflows.types.core import EntityInputsInterface, MergeBehavior
 from vellum.workflows.types.definition import (
     ComposioToolDefinition,
     DeploymentDefinition,
     MCPServer,
     MCPToolDefinition,
+    Tool,
+    ToolBase,
     VellumIntegrationToolDefinition,
 )
 from vellum.workflows.types.generics import is_workflow_class
@@ -53,6 +55,28 @@ class FunctionCallNodeMixin:
 
     function_call_output: List[PromptOutput]
 
+    def _handle_tool_exception(self, e: Exception, tool_type: str, tool_name: str) -> None:
+        """
+        Re-raise exceptions with contextual information while preserving NodeException details.
+
+        Args:
+            e: The caught exception
+            tool_type: Type of tool (e.g., "function", "MCP tool", "Vellum Integration tool")
+            tool_name: Name of the tool that failed
+        """
+        if isinstance(e, NodeException):
+            # Preserve original error code and raw_data while adding context
+            raise NodeException(
+                message=f"Error executing {tool_type} '{tool_name}': {e.message}",
+                code=e.code,
+                raw_data=e.raw_data,
+            ) from e
+        else:
+            raise NodeException(
+                message=f"Error executing {tool_type} '{tool_name}': {str(e)}",
+                code=WorkflowErrorCode.NODE_EXECUTION,
+            ) from e
+
     def _extract_function_arguments(self) -> dict:
         """Extract arguments from function call output."""
         current_index = getattr(self, "state").current_prompt_output_index
@@ -87,7 +111,7 @@ class FunctionCallNodeMixin:
 
 
 class ToolPromptNode(InlinePromptNode[ToolCallingState]):
-    max_prompt_iterations: Optional[int] = 5
+    max_prompt_iterations: Optional[int] = 25
 
     class Trigger(InlinePromptNode.Trigger):
         merge_behavior = MergeBehavior.AWAIT_ATTRIBUTES
@@ -199,11 +223,7 @@ class FunctionNode(BaseNode[ToolCallingState], FunctionCallNodeMixin):
         try:
             result = self.function_definition(**arguments)
         except Exception as e:
-            function_name = self.function_definition.__name__
-            raise NodeException(
-                message=f"Error executing function '{function_name}': {str(e)}",
-                code=WorkflowErrorCode.NODE_EXECUTION,
-            )
+            self._handle_tool_exception(e, "function", self.function_definition.__name__)
 
         # Add the result to the chat history
         self._add_function_result_to_chat_history(result, self.state)
@@ -230,10 +250,7 @@ class ComposioNode(BaseNode[ToolCallingState], FunctionCallNodeMixin):
             else:
                 result = composio_service.execute_tool(tool_name=self.composio_tool.action, arguments=arguments)
         except Exception as e:
-            raise NodeException(
-                message=f"Error executing Composio tool '{self.composio_tool.action}': {str(e)}",
-                code=WorkflowErrorCode.NODE_EXECUTION,
-            )
+            self._handle_tool_exception(e, "Composio tool", self.composio_tool.action)
 
         # Add result to chat history
         self._add_function_result_to_chat_history(result, self.state)
@@ -253,10 +270,7 @@ class MCPNode(BaseNode[ToolCallingState], FunctionCallNodeMixin):
             mcp_service = MCPService()
             result = mcp_service.execute_tool(tool_def=self.mcp_tool, arguments=arguments)
         except Exception as e:
-            raise NodeException(
-                message=f"Error executing MCP tool '{self.mcp_tool.name}': {str(e)}",
-                code=WorkflowErrorCode.NODE_EXECUTION,
-            )
+            self._handle_tool_exception(e, "MCP tool", self.mcp_tool.name)
 
         # Add result to chat history
         self._add_function_result_to_chat_history(result, self.state)
@@ -271,20 +285,31 @@ class VellumIntegrationNode(BaseNode[ToolCallingState], FunctionCallNodeMixin):
 
     def run(self) -> Iterator[BaseOutput]:
         arguments = self._extract_function_arguments()
+        vellum_client = self._context.vellum_client
 
         try:
-            vellum_service = VellumIntegrationService()
+            vellum_service = VellumIntegrationService(vellum_client)
             result = vellum_service.execute_tool(
-                integration=self.vellum_integration_tool.integration,
+                integration=self.vellum_integration_tool.integration_name,
                 provider=self.vellum_integration_tool.provider.value,
                 tool_name=self.vellum_integration_tool.name,
                 arguments=arguments,
             )
+        except NodeException as e:
+            error_payload = {
+                "error": {
+                    "code": e.code.value,
+                    "message": e.message,
+                }
+            }
+            if e.raw_data is not None:
+                error_payload["error"]["raw_data"] = e.raw_data
+
+            self._add_function_result_to_chat_history(error_payload, self.state)
+            yield from []
+            return
         except Exception as e:
-            raise NodeException(
-                message=f"Error executing Vellum Integration tool '{self.vellum_integration_tool.name}': {str(e)}",
-                code=WorkflowErrorCode.NODE_EXECUTION,
-            )
+            self._handle_tool_exception(e, "Vellum Integration tool", self.vellum_integration_tool.name)
 
         # Add result to chat history
         self._add_function_result_to_chat_history(result, self.state)
@@ -399,8 +424,8 @@ def create_router_node(
         # Avoid using lambda to capture function_name
         # lambda will capture the function_name by reference,
         # and if the function_name is changed, the port_condition will also change.
-        def create_port_condition(fn_name):
-            return Port.on_if(
+        def create_port_condition(fn_name, is_first):
+            condition = (
                 ToolCallingState.current_prompt_output_index.less_than(tool_prompt_node.Outputs.results.length())
                 & tool_prompt_node.Outputs.results[ToolCallingState.current_prompt_output_index]["type"].equals(
                     "FUNCTION_CALL"
@@ -409,26 +434,33 @@ def create_router_node(
                     "name"
                 ].equals(fn_name)
             )
+            # First port should be on_if, subsequent ports should be on_elif
+            return Port.on_if(condition) if is_first else Port.on_elif(condition)
 
+        is_first_port = True
         for function in functions:
             if isinstance(function, ComposioToolDefinition):
                 function_name = get_function_name(function)
-                port = create_port_condition(function_name)
+                port = create_port_condition(function_name, is_first_port)
                 setattr(Ports, function_name, port)
+                is_first_port = False
             elif isinstance(function, VellumIntegrationToolDefinition):
                 function_name = get_function_name(function)
-                port = create_port_condition(function_name)
+                port = create_port_condition(function_name, is_first_port)
                 setattr(Ports, function_name, port)
+                is_first_port = False
             elif isinstance(function, MCPServer):
                 tool_functions: List[MCPToolDefinition] = compile_mcp_tool_definition(function)
                 for tool_function in tool_functions:
                     name = get_mcp_tool_name(tool_function)
-                    port = create_port_condition(name)
+                    port = create_port_condition(name, is_first_port)
                     setattr(Ports, name, port)
+                    is_first_port = False
             else:
                 function_name = get_function_name(function)
-                port = create_port_condition(function_name)
+                port = create_port_condition(function_name, is_first_port)
                 setattr(Ports, function_name, port)
+                is_first_port = False
 
         # Add the else port for when no function conditions match
         setattr(Ports, "default", Port.on_else())
@@ -531,6 +563,8 @@ def create_function_node(
 
                 return func(**merged_kwargs)
 
+            # Preserve the original function name for better error messages
+            wrapper.__name__ = func.__name__
             return wrapper
 
         node = type(
